@@ -1,3 +1,23 @@
+/*
+   
+        (c) 2006 by Thomas Michel <tom.michel@arcor.de>
+
+        Kwlan is free software; you can redistribute it and/or modify
+        it under the terms of the GNU Library General Public License as
+        published by the Free Software Foundation; either version 2 of
+        the License, or (at your option) any later version.
+
+        Kwlan is distributed in the hope that it will be useful,
+        but WITHOUT ANY WARRANTY; without even the implied warranty of
+        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+        GNU Library General Public License for more details.
+
+        You should have received a copy of the GNU Library General Public License
+        along with this library; see the file COPYING.LIB.  If not, write to
+        the Free Software Foundation, Inc., 51 Franklin Steet, Fifth Floor,
+        Boston, MA 02110-1301, USA.
+*/
+
 #include <stdlib.h>
 
 #include "kprofileconfigdlg.h"
@@ -159,7 +179,7 @@ void KProfileConfigDlg::paramsFromConfig(KwlanInterface *interface, QString ssid
     int i, counter;
     QString ipaddress, netmask,gateway,dns1,dns2,domain;
     QStringList dnsSearchlist;
-    bool useDhcp, dontOverrideIp,dontOverrideDns;
+    bool useDhcp, dontOverrideIp,dontOverrideDns, hidden;
 
     m_editNetworkID = interface->getWpaId( ssid);
     if (m_editNetworkID==-1 ) m_newNetwork = TRUE;
@@ -171,6 +191,29 @@ void KProfileConfigDlg::paramsFromConfig(KwlanInterface *interface, QString ssid
     leSSID->setEnabled(FALSE);
     if (m_interface->getWpaStarted())
     {
+        snprintf(cmd, sizeof(cmd), "GET_NETWORK %d scan_ssid", m_editNetworkID);
+        reply_len = sizeof(reply);
+        int scan = 0;
+        if (interface->ctrlRequest(cmd, reply, &reply_len) >= 0) {
+            reply[reply_len] = '\0';
+            if (strstr(reply, "1"))
+                cbHiddenSsid->setChecked(TRUE);
+            else cbHiddenSsid->setChecked(FALSE);
+        }
+        
+        snprintf(cmd, sizeof(cmd), "GET_NETWORK %d mode", m_editNetworkID);
+        reply_len = sizeof(reply);
+        if (interface->ctrlRequest(cmd, reply, &reply_len) >= 0) {
+            reply[reply_len] = '\0';
+            if (strstr(reply, "0"))
+            {
+                rbInfrastructure->setChecked(TRUE);
+            }
+            else  {
+                rbAdhoc->setChecked(TRUE);
+            } 
+        }
+
         snprintf(cmd, sizeof(cmd), "GET_NETWORK %d proto", m_editNetworkID);
         reply_len = sizeof(reply);
         int wpa = 0;
@@ -390,20 +433,30 @@ int KProfileConfigDlg::setNetworkParam( int id, const char * field, const char *
     size_t reply_len;
     snprintf(cmd, sizeof(cmd), "SET_NETWORK %d %s %s%s%s",id, field, quote ? "\"" : "", value, quote ? "\"" : "");
     reply_len = sizeof(reply);
+    if (::debugOutput) kdDebug() <<  cmd << endl;
     m_interface->ctrlRequest(cmd, reply, &reply_len);
+    reply[reply_len] = '\0';
+    if (::debugOutput) kdDebug() <<  reply << endl;
     return strncmp(reply, "OK", 2) == 0 ? 0 : -1;
 }
 
 
-void KProfileConfigDlg::paramsFromScanResults(QListViewItem * sel  )
+void KProfileConfigDlg::paramsFromScanResults(kwlNetwork p  )
 {
     m_newNetwork = true;
 
     /* SSID BSSID frequency signal flags */
-    setCaption(sel->text(0));
-    leSSID->setText(sel->text(0));
-    
-    QString flags = sel->text(4);
+    if (p.m_ssid=="<hidden>")
+    {
+        leSSID->setText("");
+        cbHiddenSsid->setChecked(TRUE);
+    }
+    else {
+        leSSID->setText(p.m_ssid);
+        cbHiddenSsid->setChecked(FALSE);
+    }
+    if (p.m_adhoc) rbAdhoc->setChecked(TRUE);
+    QString flags = p.m_flags;
     int auth, encr = 0;
     if (flags.find("[WPA2-EAP") >= 0)
         auth = AUTH_WPA2_EAP;
@@ -516,6 +569,9 @@ void KProfileConfigDlg::addNetwork()
         m_editNetworkID = id;
 
         setNetworkParam(id, "ssid", leSSID->text().ascii(), true);
+        if (rbInfrastructure->isChecked())
+            setNetworkParam(id, "mode", "0", false);
+        else setNetworkParam(id, "mode", "1", false);
     
         char *key_mgmt = NULL, *proto = NULL, *pairwise = NULL;
         switch (auth) {
@@ -585,6 +641,9 @@ void KProfileConfigDlg::addNetwork()
             if (lePhase2->isEnabled())
                 if (!lePhase2->text().isEmpty())
                     setNetworkParam(id, "phase2", lePhase2->text().ascii(), true);
+            if (cbHiddenSsid->isChecked())
+                setNetworkParam(id, "scan_ssid", "1", FALSE);
+            else setNetworkParam(id, "scan_ssid", "0", FALSE);
             writeWepKey(id, leWep0, 0);
             writeWepKey(id, leWep1, 1);
             writeWepKey(id, leWep2, 2);
