@@ -1,5 +1,6 @@
 /* KwlanInterface defines a class for configuring and monitoring an interface
         Parts of it's implemtation are based on KNemo by Percy Leonhardt <percy@eris23.de>
+        The scanning part is derived from wireless-tools ba Jean Tourrilhes <jt@hpl.hp.com>
    
         (c) 2006 by Thomas Michel <tom.michel@arcor.de>
 
@@ -28,12 +29,11 @@
 #include "kstartsupplicantdlg.h"
 #include "kwlansuprocess.h"
 #include "keventhistorydlg.h"
-#include "kprofilesdlg.h"
-#include "kscandlg.h"
 #include "kprofileconfigdlg.h"
 #include "kwlansettingsdlg.h"
+#include "kscandlg.h"
 #include "globals.h"
-
+#include "kwlnetworklist.h"
 
 #include <kmessagebox.h>
 #include <ktempfile.h>
@@ -52,10 +52,12 @@
 #include <linux/if.h>
 #include <sys/ioctl.h>
 
-KwlanInterface::KwlanInterface()
+
+KwlanInterface::KwlanInterface(KwlanMainWin *mainWin)
 {
     // initialise variables
-    //Notifier = NULL;
+    //Notifier = NULL
+    m_mainWin = mainWin;
     m_controlConnection = NULL;
     m_monitorConnection = NULL;
     m_controlInterfaceDir = strdup ("/var/run/wpa_supplicant");
@@ -79,6 +81,8 @@ KwlanInterface::KwlanInterface()
     m_scanDlg = 0L;
     m_trayWin = 0L;
     m_fdSock = -1;
+    m_state = DISABLED;
+    m_isScanning = FALSE;
     KMessageBox::setDontShowAskAgainConfig(mainConfiguration().m_configuration);
     //get path to wpa_supplicant
     m_wpaPath = getPath("wpa_supplicant");
@@ -104,6 +108,7 @@ KwlanInterface::KwlanInterface()
     m_iwconfigPath = getPath("iwconfig");
     if (m_iwconfigPath.isEmpty())
         kdDebug() << i18n("iwconfig not found. Wireless statistics might not work.") << endl;
+    m_timer = new QTimer(this);
 
 }
 
@@ -159,16 +164,16 @@ void KwlanInterface::init(QString interface)
         }
     }
     // To avoid Getting an IP Address if interface already has one, check ip address
-    // We set m_ipConfigured to true to avoid checkConfig getting an ip address
+    // We set m_ipConfigured to true to avoid updateData getting an ip address
     // It's a little ugly and needs some rework
     m_ipConfigured = TRUE;
-    checkConfig();
+    updateData();
     if (m_interfaceData.ipAddress.isEmpty() ||  m_interfaceData.ipAddress=="0.0.0.0") m_ipConfigured = FALSE;
     else m_ipConfigured = TRUE;
     if (::debugOutput) kdDebug() << "IP Address " << m_interfaceData.ipAddress  << "M_ipconfigured " << m_ipConfigured << endl;
 
     //check for existing  WPA connection
-    checkConnection();
+    checkWpaConnection();
     //Start WPA Supplicant if configured and no connection is established
     if (m_configuration)
     {
@@ -223,18 +228,24 @@ void KwlanInterface::init(QString interface)
         if (m_staticIpAddress!=m_interfaceData.ipAddress)
         {
             m_ipConfigured = FALSE;
-            checkConfig();
+            updateData();
         }
     }
     emit (networksChanged());
+    slotScanAvailableNetworks();
     connect(this, SIGNAL(wpaStatusChanged( bool )), this, SLOT(slotWpaStatusChanged(bool)));
     m_timer = new QTimer(this);
+    m_scanTimer = new QTimer(this);
     int timerInterval;
     timerInterval = 100;
     if (m_timer){
-        connect (m_timer, SIGNAL(timeout()),SLOT(checkConnection()));
+        connect (m_timer, SIGNAL(timeout()),SLOT(checkWpaConnection()));
         m_timer->start(timerInterval,FALSE);
     }
+    if (m_scanTimer){
+        connect (m_scanTimer, SIGNAL(timeout()),SLOT(slotScanAvailableNetworks()));
+        m_scanTimer->start(3000,FALSE);
+    }
 
 }
         
@@ -300,17 +311,30 @@ int KwlanInterface::openControlConnection()
         m_monitorConnection = NULL;
     }
 
-    //kdDebug() << "Trying to connect to " <<  cfile << endl;
+    //if (::debugOutput) kdDebug() << "Trying to connect to " <<  cfile << endl;
     m_controlConnection = wpa_ctrl_open(cfile);
     if (m_controlConnection == NULL) {
-        //kdDebug() << "Opening control connection failed" << endl;
+        //if (::debugOutput) kdDebug() << "Opening control connection failed" << endl;
         delete cfile;
         return -1;
     }
     m_wpaStarted = TRUE;
-    //kdDebug() << "Control connection established" << endl;
     if (!tmpWpaStarted)
     {
+        if (debugOutput) kdDebug() << "Control connection established" << endl;
+        // Set ap_scan value as new connection is established
+        char buf[10];
+        char command[100];
+        size_t len;
+        len = sizeof(buf)-1;
+        snprintf(command,sizeof(command),"AP_SCAN %d",m_configuration->m_ap_scan);
+        if (::debugOutput) kdDebug() << "Setting ap_scan value: " << command << endl;
+        if (ctrlRequest(command,buf,&len) <0)
+        {
+            kdDebug() << "Setting ap_scan value failed!" << endl;
+        }
+        buf[len] = '\0';
+        if (::debugOutput) kdDebug() << buf << endl;
         emit (wpaStatusChanged(m_wpaStarted));
         emit (networksChanged());
     }
@@ -318,7 +342,7 @@ int KwlanInterface::openControlConnection()
     delete cfile;
     if (m_monitorConnection == NULL) {
         wpa_ctrl_close(m_controlConnection);
-        //kdDebug() << "Could not establish monitor connection." << endl;
+        if (::debugOutput) kdDebug() << "Could not establish monitor connection." << endl;
         return -1;
     }
     if (wpa_ctrl_attach(m_monitorConnection)) {
@@ -338,7 +362,7 @@ int KwlanInterface::openControlConnection()
     
 }
 
-void KwlanInterface::checkConnection()
+void KwlanInterface::checkWpaConnection()
 {
     char buf[10];
     size_t len;
@@ -365,7 +389,7 @@ void KwlanInterface::checkConnection()
                 emit wpaStatusChanged (TRUE);
         }
     }
-    checkConfig();
+    updateData();
 }
 
 int KwlanInterface::ctrlRequest(const char *command, char *buf, size_t *buflen)
@@ -447,7 +471,7 @@ QStringList KwlanInterface::listNonWpaProfiles(QStringList wpaProfiles)
     QStringList profiles;
     if (!m_configuration) return profiles;
     profiles = m_configuration->getNetworks();
-    // Now test if networks already esist from wpa config
+    // Now test if networks already exists from wpa config
     QStringList::Iterator it;
     it = profiles.begin();
     while (it !=profiles.end())
@@ -466,7 +490,7 @@ QStringList KwlanInterface::listNonWpaProfiles(QStringList wpaProfiles)
         }
         it++;
     }
-    kdDebug() << profiles << endl;
+    //kdDebug() << profiles << endl;
     return profiles;
 }
 
@@ -543,6 +567,7 @@ bool KwlanInterface::profileActivateNonWpa(QString ssid)
         KwlanSuProcess *iwconfig = new KwlanSuProcess(this);
         *iwconfig << m_iwconfigPath << m_controlInterface << QString("essid") << ssid;
         iwconfig->setDescription(i18n("Configure wlan interface using iwconfig"));
+        //iwconfig->start(SuProcessBase::Block);
         iwconfig->start();
     }
    // store in config
@@ -558,6 +583,7 @@ bool KwlanInterface::profileActivateNonWpa(QString ssid)
 bool KwlanInterface::profileActivateWpa(QString ssid)
 {
         // select network using wpa_supplicant
+    if (!m_wpaStarted) return FALSE;
     int ret;
     char reply[40], cmd[256];
     size_t reply_len = sizeof(reply);
@@ -592,7 +618,7 @@ bool KwlanInterface::profileActivateWpa(QString ssid)
     m_ssid = ssid;
     getIpData();
     
-    checkConnection();
+    checkWpaConnection();
     return TRUE;   
 }
 
@@ -600,9 +626,9 @@ bool KwlanInterface::profileActivate(QString ssid)
 {
     if (!m_configureInterface) return FALSE;
     // release any existing ip address
-    if (m_interfaceData.connected) releaseIpAddress();
+    if (m_ipConfigured) releaseIpAddress();
     //m_interfaceData.available = FALSE;
-    //m_interfaceData.connected = FALSE;
+    m_interfaceData.connected = FALSE;
     if (m_controlConnection){
         return profileActivateWpa(ssid);
     }
@@ -627,7 +653,7 @@ void KwlanInterface::releaseIpAddress()
             KwlanSuProcess *proc = new KwlanSuProcess();
             *proc << script;
             proc->setDescription( i18n("start script before disconnect"));
-            proc->start( );
+            proc->start();
         }
         else {
             KProcess *proc = new KProcess();
@@ -649,12 +675,13 @@ void KwlanInterface::releaseIpAddress()
     }
     m_releaseProcess->setDescription(i18n("Release IP address"));
     connect (m_releaseProcess, SIGNAL(processExited(KwlanSuProcess*)), this, SLOT(releaseProcessExited(KwlanSuProcess*)));
+    //m_releaseProcess->start(SuProcessBase::Block);
     m_releaseProcess->start();
     //m_interfaceData.ipAddress="";
     //emit ipChanged( QString(""));
 }
 
-void KwlanInterface::setNetworkSettings()
+void KwlanInterface::setDnsSettings()
 {
     if (m_type == PPP) return;
     if (m_staticDns1.isEmpty() && m_staticDns2.isEmpty() && m_domain.isEmpty() && m_dnsSearchlist.isEmpty())
@@ -684,6 +711,7 @@ void KwlanInterface::setNetworkSettings()
         *setNetwork << ";";
         *setNetwork << getPath("chown") << "root:root" << "/etc/resolv.conf";
         setNetwork->setDescription(i18n("Set network settings"));
+        //setNetwork->start(SuProcessBase::Block);
         setNetwork->start();
     }
 }
@@ -740,16 +768,17 @@ void KwlanInterface::getIpAddress()
         } else if (m_useDhcpcd){
             *m_dhcpProcess << m_dhcpcdPath << "-nd" << m_controlInterface;
         }
-	
+        say(i18n("%1 is requesting\nan IP address").arg(m_controlInterface));
         m_dhcpProcess->setDescription(i18n("Obtain IP address dynamically"));
         m_dhcpProcess->setRestart(true);
     }
     connect (m_dhcpProcess,SIGNAL(processExited(KwlanSuProcess*)), this, SLOT(dhcpProcessExited(KwlanSuProcess*)));
+    //m_dhcpProcess->start(SuProcessBase::Block);
     m_dhcpProcess->start();
 
 }
 
-void KwlanInterface::processMessage(char *message)
+void KwlanInterface::processWpaMessage(char *message)
 {
     char *pos = message, *pos2;
     int priority = 2;
@@ -789,13 +818,7 @@ void KwlanInterface::processMessage(char *message)
         if (str_match(pos, WPA_CTRL_REQ))
             processCtrlReq(pos + strlen(WPA_CTRL_REQ));
         if (str_match(pos,WPA_EVENT_CONNECTED)){
-        // getIpAddress
-        //    if (!m_associated) {
-        //        getIpAddress();
-        //    }
             m_associated = true;
-            //kdDebug() << "Connection established." << endl;
-            //emit connectionStatusChanged( TRUE);
         }
         if (str_match(pos,WPA_EVENT_DISCONNECTED)){
             m_associated = false;
@@ -811,7 +834,7 @@ void KwlanInterface::receiveMsgs()
         len = sizeof(buf) - 1;
         if (wpa_ctrl_recv(m_monitorConnection, buf, &len) == 0) {
             buf[len] = '\0';
-            processMessage(buf);
+            processWpaMessage(buf);
         }
     }
 }
@@ -878,9 +901,6 @@ void KwlanInterface::stopWpa()
         return;
     }
     if (::debugOutput) kdDebug() << "wpa_supplicant stopped" << endl;
-    //m_wpaStarted = FALSE;
-    //m_controlConnection = 0L;
-    //emit (wpaStatusChanged(FALSE));
     emit (networksChanged());
 }
 
@@ -895,7 +915,7 @@ void KwlanInterface::profileAdded()
     emit (networksChanged());
 }
 
-void KwlanInterface::checkConfig()
+void KwlanInterface::updateData()
 {
     bool tmpAvailable = m_interfaceData.available;
     bool tmpUp = m_interfaceData.up;
@@ -915,10 +935,12 @@ void KwlanInterface::checkConfig()
         fclose(flags_fp);
         currentStatus = IFF_UP & flags;
     }
-    if (!currentStatus && m_interfaceData.up) { // interface down...
+    //if (!currentStatus && m_interfaceData.up) { // interface down...
+    if (!currentStatus) { // interface down...
         // Interface is down and was up before -> set status variables
         m_interfaceData.up = FALSE;
         m_interfaceData.connected = FALSE;
+        m_state = DISABLED;
     } else if (currentStatus) {
         // Interface is up, so check for connection status
         m_interfaceData.up = TRUE;
@@ -930,7 +952,8 @@ void KwlanInterface::checkConfig()
             fclose(carrier_fp);
         }
         if (carrierFlag == '0') { // carrier down
-            // Interface is not connected, so return;
+            // Interface is not connected
+            m_state=NOTCONNECTED;
             if (m_interfaceData.connected) {
                 m_interfaceData.connected = FALSE;
                 say(i18n("%1 is disconnected").arg(m_controlInterface));
@@ -938,7 +961,14 @@ void KwlanInterface::checkConfig()
         } else if (!m_interfaceData.connected) { 
             // Interface is connected
             m_interfaceData.connected = TRUE;
-            say(i18n("%1 is connected").arg(m_controlInterface));
+            if (m_ipConfigured)
+            {
+                m_state=CONFIGURED;
+                say(i18n("%1 is connected").arg(m_controlInterface));
+            }
+            else {
+                m_state=CONNECTED;
+            }
         }
         if (m_interfaceData.wirelessDevice)
         {
@@ -1054,9 +1084,16 @@ void KwlanInterface::checkConfig()
     //if (tmpAvailable != m_interfaceData.available) 
     //    emit (connectionStatusChanged (m_interfaceData.available));
     if (tmpUp != m_interfaceData.up)
+    {
         emit (interfaceUp(m_interfaceData.up ));
-    if (m_interfaceData.up && m_interfaceData.connected && !m_ipConfigured)
-        getIpAddress();
+    }
+    if (m_interfaceData.up && m_interfaceData.connected)
+    {
+        if (!m_ipConfigured)
+        {
+            getIpAddress();
+        }
+    }
 }
 
 
@@ -1065,13 +1102,19 @@ void KwlanInterface::dhcpProcessExited(KwlanSuProcess *process)
     if (process == m_dhcpProcess)
     {
         delete m_dhcpProcess;    
+        // check if the process returned an Error
+        if (m_dhcpProcess->exitStatus()==0)
+        {
+            say(i18n("%1 is connected").arg(m_controlInterface));
+            if (::debugOutput) kdDebug() << "Now starting script after connect" << endl;
+            // set Network settings like DNS etc
+            setDnsSettings();
+            //start script after connect
+            startScriptAfterConnect();
+        }
         m_ipConfigured = TRUE;
+        m_state=CONFIGURED;
         m_dhcpProcess =0L;
-        if (::debugOutput) kdDebug() << "Now starting script after connect" << endl;
-        // set Network settings like DNS etc
-        setNetworkSettings();
-        //start script after connect
-        startScriptAfterConnect();
     }
 }
 
@@ -1347,41 +1390,21 @@ void KwlanInterface::enableInterface( bool enable)
     if (enable) {
         *ifup << m_ifconfigPath << m_controlInterface  << "up";
         ifup->setDescription( i18n("Enable interface"));
-        ifup->start( );
+        //ifup->start(SuProcessBase::Block );
+        ifup->start();
     }
     else if (m_type != PPP){
-        releaseIpAddress(); // Don't release IP Address as it will reenable the interface if 
+        //releaseIpAddress(); // Don't release IP Address as it will reenable the interface if 
         // it is run after setting the interface to down state.
+        // TODO: Release IP Address and connect processExited with function to set interface down
         *ifup << m_ifconfigPath << m_controlInterface << "down";
         ifup->setDescription( i18n("Disable interface"));
-        ifup->start( );
+        //ifup->start( SuProcessBase::Block );
+        ifup->start();
     }
 }
 
 
-void KwlanInterface::editProfiles()
-{
-    KProfilesDlg *dlg = new KProfilesDlg();
-    dlg->setInterface( this);
-    dlg->show();
-    dlg->exec();
-}
-
-void KwlanInterface::slotScan()
-{
-    if (m_scanDlg) {
-        m_scanDlg->close();
-        delete m_scanDlg;
-    }
-
-    m_scanDlg = new KScanDlg();
-    if (m_scanDlg == NULL)
-        return;
-    m_scanDlg->setInterface(this);
-    m_scanDlg->show();
-    m_scanDlg->exec();
-}
-
 void KwlanInterface::slotNewProfile()
 {
     KProfileConfigDlg *profileDlg = new KProfileConfigDlg();
@@ -1532,4 +1555,72 @@ void KwlanInterface::slotEnableConfigureInterface(QString interface, bool enable
 {
     if (interface != QString(m_controlInterface)) return;
     m_configureInterface = enable;
-}
\ No newline at end of file
+}
+
+void KwlanInterface::slotScan()
+{
+    if (m_scanDlg) {
+        m_scanDlg->close();
+        delete m_scanDlg;
+    }
+
+    m_scanDlg = new KScanDlg();
+    if (m_scanDlg == NULL)
+        return;
+    m_scanDlg->setInterface(this);
+    m_scanDlg->show();
+    m_scanDlg->exec();
+}
+
+void KwlanInterface::slotScanAvailableNetworks()
+{
+    //avoid mulitple scans due timeout of scann being longer than scan interval
+    if (m_isScanning | (!m_interfaceData.wirelessDevice)) return;
+    m_isScanning = TRUE;
+    QStringList configuredProfiles = listProfiles();
+    char buf[2048]; QString signal;
+    m_availableNetworks.clear();
+    size_t len =sizeof(buf);
+    // Are we connected to wpa? If so, send scan request
+    // We are not using the results but without it the iwlib functions 
+    // will not return any network;
+    if (m_controlConnection) {
+        if (ctrlRequest("SCAN", buf, &len) < 0)
+        {
+            kdDebug() << "Could not scan for wpa available networks" << endl;
+        }
+    }
+    
+    kwireless_scan(m_availableNetworks,m_controlInterface);
+    // check if found networks are already configured
+    for (kwlNetworkList::iterator it = m_availableNetworks.begin();it != m_availableNetworks.end();it++)
+    {
+        QStringList::Iterator index = configuredProfiles.find((*it).m_ssid);
+        if ((*index).isEmpty())
+        {
+            (*it).m_isConfigured = FALSE;
+        } else (*it).m_isConfigured = TRUE;
+   
+    }
+    m_isScanning = FALSE;
+    return;
+
+}
+
+kwlNetworkList KwlanInterface::getAvailableProfiles()
+{
+    QStringList profiles = listProfiles();
+    if ( !m_interfaceData.wirelessDevice )
+    {
+        kwlNetworkList tmpList;
+        for (QStringList::Iterator it = profiles.begin(); it != profiles.end();it++)
+        {
+            tmpList.append(kwlNetwork(*it));
+        }
+        return tmpList;
+    }
+    return m_availableNetworks;
+}
+
+
+
